package examples;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.net.InetAddress;
import java.net.UnknownHostException;

import javax.swing.InputMap;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.text.DefaultEditorKit;

import comm.Connector;
import comm.MyMessage;

public class ExpertChat {
	private JFrame main;
	private Connector connector;
	private JTextField textReceiverHost;
	private JTextField textReceiverPort;
	private JTextArea textMessage;
	private JButton buttonClear;
	private JButton buttonSend;
	private JPanel panelCampos;
	private JTextArea textArea;
	private int port;
	private int messageId;

	public ExpertChat() {
		//port = 4400;
		init();
	}

	public ExpertChat(int port) {
		this.port = port;
		init();
	}

	private void init() {
		messageId = 0;
		connector = new Connector(port);
		main = new JFrame("ExpertChatter " + getHost() + ":" + port);
		main.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		main.setLayout(new BorderLayout());

		panelCampos = new JPanel(new GridLayout(4, 2));
		panelCampos.add(buttonClear = new JButton("Clear"));
		panelCampos.add(buttonSend = new JButton("Send"));
		panelCampos.add(new JLabel("Host"));
		panelCampos.add(textReceiverHost = new JTextField(20));
		panelCampos.add(new JLabel("Port"));
		panelCampos.add(textReceiverPort = new JTextField());
		panelCampos.add(new JLabel("Message"));

		buttonClear.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				textReceiverHost.setText("");
				textReceiverPort.setText("");
				textMessage.setText("");
				textReceiverHost.grabFocus();
			}
		});
		buttonSend.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				sendMessage();
			}
		});

		main.add(panelCampos, BorderLayout.NORTH);
		main.add(new JScrollPane(textMessage = new JTextArea(3, 40)),
				BorderLayout.CENTER);
		main.add(new JScrollPane(textArea = new JTextArea(20, 10)),
				BorderLayout.SOUTH);
		
		// Ao pressionar a tecla enter no textMessage, da um backspace e volta.
		InputMap inputMap = textMessage.getInputMap();  
		// quando teclar ENTER volta 1 espaco sem apagar nada.  
		KeyStroke key = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_ENTER,0,false);  
		inputMap.put(key, DefaultEditorKit.backwardAction); 
		//Se clicar no enter, envia a mensagem
		textMessage.addKeyListener(new KeyListener() {
			public void keyPressed(KeyEvent e) {
				if (e.getKeyChar() == KeyEvent.VK_ENTER) {
					sendMessage();
				}
			}

			// unused abstract methods
			public void keyTyped(KeyEvent e) {
			}

			public void keyReleased(KeyEvent e) {
			}
		});
		textArea.setEditable(false);
		main.pack();
		main.setVisible(true);
		textReceiverHost.grabFocus();
		run();
	}

	private void addLog(String msg) {
		textArea.setText(msg + "\n" + textArea.getText());
		textArea.setCaretPosition(0);
	}

	private void sendMessage() {
		if (textReceiverHost.getText().equals("")
				|| textReceiverPort.getText().equals("")) {
			JOptionPane.showMessageDialog(null, "Please, set host and port.");
			if (textReceiverHost.getText().equals("")) {
				textReceiverHost.grabFocus();
			} else {
				textReceiverPort.grabFocus();
			}
		} else {
			MyMessage out = new MyMessage();
			// Sender
			out.add(getHost());
			out.add(Integer.toString(this.port));
			out.add(textReceiverHost.getText());
			// Receiver
			out.add(textReceiverPort.getText());
			out.add(Integer.toString(messageId));
			out.add(textMessage.getText());
			// Message
			connector.sendMessage(textReceiverHost.getText(), Integer
					.parseInt(textReceiverPort.getText()), out.toString());
			String log = "[" + textReceiverHost.getText() + ":"
					+ textReceiverPort.getText() + "] " + textMessage.getText();
			addLog("send(" + messageId + "): " + log);
			textMessage.setText("");
			messageId++;
		}
	}
	
	private String getHost() {
		String hostaddress = "";
		try {
			InetAddress addr = InetAddress.getLocalHost();
			// Get IP Address
			// byte[] ipAddr = addr.getAddress();
			// Get hostname
			// String hostname = addr.getHostName();
			// Get hostaddress (ip) as String
			hostaddress = addr.getHostAddress();
		} catch (UnknownHostException e) {
			e.printStackTrace();
		}
		return hostaddress;
	}

	private void run() {
		while (true) {
			if (connector.hasMessage()) {
				MyMessage in = new MyMessage(connector.getMessage());
				String senderHost = in.next();
				String senderPort = in.next();
				String receiverHost = in.next(); // unused
				String receiverPort = in.next(); // unused
				String id = in.next();
				String content = in.next();
				String log = "recv(" + id + "): [" + senderHost + ":" + senderPort + "] " + content;
				addLog(log);
			}
		}
	}

	public static void main(String[] arguments) {
		if (arguments.length == 0) {
			new ExpertChat();
		} else {
			new ExpertChat(Integer.parseInt(arguments[0]));
		}
	}
}
